#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_MLX90614.h>
#include <VL53L0X.h>
VL53L0X sensor;

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64

#define OLED_RESET    -1
#define SCREEN_ADDRESS 0x3C

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

Adafruit_MLX90614 mlx = Adafruit_MLX90614();

float emissivity = 0.98;

int readingButton2 = 1;
bool lastButtonState = HIGH;
bool buttonReleased = false;
unsigned long buttonPressStartTime = 0;
bool buttonState = HIGH;

bool mode = false; // false = peau, true = objets

float emissivityObject = 1.0;

const int buttonPin = 5;  // Broche du bouton
const int buzzerPin = 4;  // Broche du buzzer

bool measurementTaken = false;  // Indicateur de prise de mesure
float temperature_interne = 0;
float temperature_front = 0;

void setup() {
  Serial.begin(9600);
  Serial.println("Initializing...");

  if (!mlx.begin()) {
    Serial.println("Failed to find MLX sensor, check wiring!");
    while (1);
  }

  Wire.begin();

  sensor.setTimeout(500);
  if (!sensor.init())
  {
    Serial.println("Failed to detect and initialize sensor!");
    while (1) {}
  }

#if defined LONG_RANGE
  // lower the return signal rate limit (default is 0.25 MCPS)
  sensor.setSignalRateLimit(0.1);
  // increase laser pulse periods (defaults are 14 and 10 PCLKs)
  sensor.setVcselPulsePeriod(VL53L0X::VcselPeriodPreRange, 18);
  sensor.setVcselPulsePeriod(VL53L0X::VcselPeriodFinalRange, 14);
#endif

#if defined HIGH_SPEED
  // reduce timing budget to 20 ms (default is about 33 ms)
  sensor.setMeasurementTimingBudget(20000);
#elif defined HIGH_ACCURACY
  // increase timing budget to 200 ms
  sensor.setMeasurementTimingBudget(200000);
#endif

  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;);
  }
  display.setRotation(2);


  pinMode(5, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
  pinMode(buzzerPin, OUTPUT);  // Configuration de la broche du buzzer en sortie
  display.setTextColor(SSD1306_WHITE);
  displayOLED();
}

void loop() {
  if (digitalRead(5) == LOW) {
    /*if (mode == 0) {
      mlx.writeEmissivity(0.98); //emissivity of human skin
      }
      else if (mode == 1) { //object mode
      mlx.writeEmissivity(emissivityObject);
      }*/


    int distance = sensor.readRangeSingleMillimeters();
    Serial.print("Distance (mm): ");
    Serial.println(distance);

    // Vérifier si la distance est comprise entre 59 et 64 mm
    if (distance >= 59 && distance <= 64 && !measurementTaken) {
      temperature_front = mlx.readObjectTempC();

      // Correction pour la température interne
      temperature_interne = temperature_front + 3.5; // À ajuster selon les besoins

      // Affichage des températures
      Serial.print("Température du front : ");
      Serial.print(temperature_front);
      Serial.println(" °C");

      Serial.print("Température interne corrigée : ");
      Serial.print(temperature_interne);
      Serial.println(" °C");

      measurementTaken = true;  // Indiquer que la mesure a été prise
      // Activation du buzzer pendant une seconde
      digitalWrite(buzzerPin, HIGH);
      delay(1000);
      digitalWrite(buzzerPin, LOW);
      displayOLED();

      // Mesure de température du front
    }
    else {
      Serial.println("Distance hors de portée");
    }
  } else {
    measurementTaken = false;  // Réinitialiser l'indicateur de prise de mesure lorsque le bouton est relâché
  }


  readingButton2 = digitalRead(6);
  /*
    // Gestion des rebonds
    if (readingButton2 != lastButtonState) {
      lastDebounceTime = millis();
    }

    if (millis() - lastDebounceTime > 50) { */
  // Si le bouton est stable depuis suffisamment longtemps
  if (readingButton2 != buttonState) {
    buttonState = readingButton2;

    // Si le bouton vient d'être relâché
    if (buttonState == LOW) {
      // Mesure de la durée d'appui sur le bouton
      buttonPressStartTime = millis();
    } else {
      unsigned long buttonPressDuration = millis() - buttonPressStartTime;

      // Si le bouton a été appuyé moins de 2 secondes
      if (buttonPressDuration < 2000) {
        if (mode == 1) {
          if (emissivityObject > 0.10) {
            emissivityObject -= 0.02;
          }
          else {
            emissivityObject = 1;
          }
        }
        displayOLED();
        Serial.print("emissivity: ");
        Serial.println(emissivityObject);
      }
      // Si le bouton a été appuyé plus de 2 secondes
      else {
        mode = !mode;
        Serial.print("Mode: ");
        Serial.println(mode);
        temperature_front = 0;
        temperature_interne = 0;
        if (mode == 0) {
          mlx.writeEmissivity(0.98); //emissivity of human skin
          displayOLED();
        }
        else {
          mlx.writeEmissivity(emissivityObject);
          displayOLED();
        }
      }
    }
    //}
  }

}

void displayOLED() {
  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);
  if (mode == 0) {
    display.print("Mode: Medical");
  }
  else {
    display.print("Mode: Object");
    display.setCursor(100, 0);
    display.print(emissivityObject);
  }
  display.setTextSize(2);
  display.setCursor(35, 30);
  if ( mode == 0) {
    if (temperature_interne == 0) {
      display.print("---");
    }
    else {
      display.print(temperature_interne, 1);
    }
  }
  else {
    if (temperature_front == 0) {
      display.print("---");
    }
    else {
      display.print(temperature_front, 1);
    }
  }
  display.print(" C");
  display.display();
  display.setTextSize(1);
  display.display();
}
